#
# urbi-doc.m4: This file is part of build-aux.
# Copyright (C) 2010, Gostai S.A.S.
#
# This software is provided "as is" without warranty of any kind,
# either expressed or implied, including but not limited to the
# implied warranties of fitness for a particular purpose.
#
# See the LICENSE file for more information.
# For comments, bug reports and feedback: http://www.urbiforge.com
#

m4_pattern_forbid([^URBI_])dnl

AC_PREREQ([2.60])

# URBI_ARG_ENABLE_DOC_SECTIONS
# ---------------------------
AC_DEFUN([URBI_ARG_ENABLE_DOC_SECTIONS],
[URBI_ARGLIST_ENABLE([enable-doc-sections=SECTIONS],
                     [generate the documentation],
                     [all|platforms|nao|spykee|nao|no|none|yes],
                     [all],
                     [
      SECTIONS: comma-separated list of:
        - all: build them all
        - platforms: generate the Platforms part
        - nao, spykee, webots:
          include the corresponding chapter in the Platforms part
          and enable the Platforms part
        - no: alias for none
        - none: build none of them
        - yes: alias for all
])

urbi_enable_doc_sections ()
{
  local section
  for section
  do
    case $section in
      (all|yes)   urbi_enable_doc_sections platforms nao spykee webots;;
      (platforms) enable_doc_section_platforms=true;;

      (nao)       urbi_enable_doc_sections platforms
                  enable_doc_section_nao=true;;

      (spykee)    urbi_enable_doc_sections platforms
                  enable_doc_section_spykee=true;;

      (webots)    urbi_enable_doc_sections platforms
                  enable_doc_section_webots=true;;

      (none|no)   enable_doc_section_platforms=false
                  enable_doc_section_nao=false
                  enable_doc_section_spykee=false
                  enable_doc_section_webots=false;;
    esac
  done
}

urbi_enable_doc_sections all $(echo $enable_doc_sections | tr ',' ' ')
])


# URBI_ENABLE_DOC_SECTIONS
# -----------------------
AC_DEFUN([URBI_ENABLE_DOC_SECTIONS],
[AC_MSG_CHECKING([for documentation sections])

URBI_ARG_ENABLE_DOC_SECTIONS

# Compute a summary for sake of AC_MSG_RESULT.
urbi_doc_sections=
for section in platforms nao spykee webots
do
  eval "ac_val=\$enable_doc_section_$section"
  if "$ac_val"; then
    urbi_doc_sections="$urbi_doc_sections $section"
  fi
done

AC_MSG_RESULT([$urbi_doc_sections])
AC_SUBST([enable_doc_section_platforms])
AC_SUBST([enable_doc_section_nao])
AC_SUBST([enable_doc_section_spykee])
AC_SUBST([enable_doc_section_webots])
])


## Local Variables:
## mode: autoconf
## End:
