/*
 * Copyright (C) 2008-2010, Gostai S.A.S.
 *
 * This software is provided "as is" without warranty of any kind,
 * either expressed or implied, including but not limited to the
 * implied warranties of fitness for a particular purpose.
 *
 * See the LICENSE file for more information.
 */

#include <libport/debug.hh>
#include <iostream>
#include <urbi/uobject.hh>

GD_ADD_CATEGORY(timer);

urbi::UObjectHub* thub;
class timer: public urbi::UObject
{
public:
  timer(const std::string& name)
    : urbi::UObject(name)
  {
    UBindVar(timer, updated);
    UBindVar(timer, timerup);
    UBindVar(timer, hupdated);
    hupdated = 0;
    timerup = 0;
    updated = 0;
    UBindFunction(timer, setupUpdate);
    UBindFunction(timer, setupTimer);
    UBindFunction(timer, unsetupTimer);
    UBindFunction(timer, setupHubUpdate);
    UBindFunction(timer, init);
  }
  int init()
  {
    return 0;
  }
  int setupUpdate(int d)
  {
    USetUpdate(d);
    return 0;
  }
  std::string setupTimer(int d)
  {
    return *USetTimer(d, &timer::onTimer);
  }
  bool unsetupTimer(const std::string& s)
  {
    return removeTimer(urbi::TimerHandle(new std::string(s)));
  }
  int setupHubUpdate(int d)
  {
    thub->USetUpdate(d);
    return 0;
  }
  virtual int update()
  {
    updated = (int)updated + 1;
    return 0;
  }
  virtual int onTimer()
  {
    timerup = (int)timerup + 1;
    return 0;
  }
  urbi::UVar updated;
  urbi::UVar hupdated;
  urbi::UVar timerup;
};

class timerHub: public urbi::UObjectHub
{
public:
  timerHub(const std::string& name)
    : urbi::UObjectHub(name)
  {
    GD_CATEGORY(timer);
    GD_INFO_DEBUG("timerhub started");
    thub = this;
  }

  virtual int update()
  {
    urbi::UVar v("timer","hupdated");
    v = 1;
    return 0;
  }
};

UStart(timer);
UStartHub(timerHub);
